@@ -1,4 +1,5 @@
 import hashlib
+import inspect
 import json
 import logging
 import os
@@ -6,17 +7,19 @@ import re
 import shutil
 import tarfile
 from collections.abc import Mapping
-from dataclasses import asdict, is_dataclass
+from dataclasses import asdict, dataclass, is_dataclass
 from enum import Enum
 from os.path import basename, isdir, isfile, join
 from pathlib import Path
-from typing import Callable, Optional, Union
+from typing import Callable, List, Optional, Union
 from urllib.parse import urlparse
 from zipfile import ZipFile, is_zipfile
 
 import requests
 from filelock import FileLock
+from huggingface_hub import HfApi, snapshot_download
 
+from .. import __version__
 from ..file_utils import get_from_cache, is_remote_url, torch_cache_home
 
 
@@ -29,10 +32,11 @@ HEAD_WEIGHTS_NAME = "pytorch_model_head.bin"
 ADAPTERFUSION_CONFIG_NAME = "adapter_fusion_config.json"
 ADAPTERFUSION_WEIGHTS_NAME = "pytorch_model_adapter_fusion.bin"
 
-ADAPTER_IDENTIFIER_PATTERN = r"[0-9a-zA-Z\-_\/@]{2,}"
 ADAPTER_HUB_URL = "https://raw.githubusercontent.com/Adapter-Hub/Hub/master/dist/v2/"
 ADAPTER_HUB_INDEX_FILE = ADAPTER_HUB_URL + "index/{}.json"
 ADAPTER_HUB_CONFIG_FILE = ADAPTER_HUB_URL + "architectures.json"
+ADAPTER_HUB_ALL_FILE = ADAPTER_HUB_URL + "all.json"
+ADAPTER_HUB_ADAPTER_ENTRY_JSON = ADAPTER_HUB_URL + "adapters/{}/{}.json"
 
 # the download cache
 ADAPTER_CACHE = join(torch_cache_home, "adapters")
@@ -55,6 +59,28 @@ class AdapterType(str, Enum):
         return self.value
 
 
+@dataclass
+class AdapterInfo:
+    """
+    Holds information about an adapter publicly available on AdapterHub or huggingface.co. Returned by
+    :func:`list_adapters()`.
+
+    Args:
+        source (str): The source repository of this adapter. Can be either "ah" (AdapterHub) or "hf" (huggingface.co).
+        adapter_id (str): The unique identifier of this adapter.
+        model_name (str, optional): The identifier of the model this adapter was trained for.
+    """
+
+    source: str
+    adapter_id: str
+    model_name: Optional[str] = None
+    task: Optional[str] = None
+    subtask: Optional[str] = None
+    username: Optional[str] = None
+    adapter_config: Optional[dict] = None
+    sha1_checksum: Optional[str] = None
+
+
 class DataclassJSONEncoder(json.JSONEncoder):
     def default(self, o):
         if is_dataclass(o):
@@ -227,7 +253,7 @@ def _dict_extract(d, primary_key, secondary_key=None):
             else:
                 for k, v in v.items():
                     yield v
-        else:
+        elif secondary_key is None:
             for k, v in v.items():
                 if k == primary_key:
                     yield v
@@ -240,6 +266,12 @@ def find_in_index(
     strict: bool = False,
     index_file: str = None,
 ) -> Optional[str]:
+    identifier = identifier.strip()
+    # identifiers of form "@<org>/<file>" are unique and can be retrieved directly
+    match = re.match(r"@(\S+)\/(\S+)", identifier)
+    if match:
+        return ADAPTER_HUB_ADAPTER_ENTRY_JSON.format(match.group(1), match.group(2))
+
     if not index_file:
         index_file = download_cached(ADAPTER_HUB_INDEX_FILE.format(model_name))
     if not index_file:
@@ -357,11 +389,23 @@ def pull_from_hub(
     return download_path
 
 
+def pull_from_hf_model_hub(specifier: str, version: str = None, **kwargs) -> str:
+    download_path = snapshot_download(
+        specifier,
+        revision=version,
+        cache_dir=kwargs.pop("cache_dir", None),
+        library_name="adapter-transformers",
+        library_version=__version__,
+    )
+    return download_path
+
+
 def resolve_adapter_path(
     adapter_name_or_path,
     model_name: str = None,
     adapter_config: Union[dict, str] = None,
     version: str = None,
+    source: str = "ah",
     **kwargs
 ) -> str:
     """
@@ -399,10 +443,61 @@ def resolve_adapter_path(
                     WEIGHTS_NAME, CONFIG_NAME, adapter_name_or_path
                 )
             )
-    # matches possible form of identifier in hub
-    elif re.fullmatch(ADAPTER_IDENTIFIER_PATTERN, adapter_name_or_path):
+    elif source == "ah":
         return pull_from_hub(
             adapter_name_or_path, model_name, adapter_config=adapter_config, version=version, **kwargs
         )
+    elif source == "hf":
+        return pull_from_hf_model_hub(adapter_name_or_path, version=version, **kwargs)
     else:
         raise ValueError("Unable to identify {} as a valid module location.".format(adapter_name_or_path))
+
+
+def list_adapters(source: str = None, model_name: str = None) -> List[AdapterInfo]:
+    """
+    Retrieves a list of all publicly available adapters on AdapterHub.ml or on huggingface.co.
+
+    Args:
+        source (str, optional): Identifier of the source(s) from where to get adapters. Can be either:
+
+            - "ah": search on AdapterHub.ml.
+            - "hf": search on HuggingFace model hub (huggingface.co).
+            - None (default): search on all sources
+
+        model_name (str, optional): If specified, only returns adapters trained for the model with this identifier.
+    """
+    adapters = []
+    if source == "ah" or source is None:
+        try:
+            all_ah_adapters_file = download_cached(ADAPTER_HUB_ALL_FILE)
+        except requests.exceptions.HTTPError:
+            raise EnvironmentError(
+                "Unable to load list of adapters from AdapterHub.ml. The service might be temporarily unavailable."
+            )
+        with open(all_ah_adapters_file, "r") as f:
+            all_ah_adapters_data = json.load(f)
+        adapters += [AdapterInfo(**info) for info in all_ah_adapters_data]
+    if source == "hf" or source is None:
+        if "fetch_config" in inspect.signature(HfApi.list_models).parameters:
+            kwargs = {"full": True, "fetch_config": True}
+        else:
+            logger.warning(
+                "Using old version of huggingface-hub package for fetching. Please upgrade to latest version for accurate results."
+            )
+            kwargs = {"full": True}
+        all_hf_adapters_data = HfApi().list_models(filter="adapter-transformers", **kwargs)
+        for model_info in all_hf_adapters_data:
+            adapter_info = AdapterInfo(
+                source="hf",
+                adapter_id=model_info.modelId,
+                model_name=model_info.config.get("adapter_transformers", {}).get("model_name")
+                if model_info.config
+                else None,
+                username=model_info.modelId.split("/")[0],
+                sha1_checksum=model_info.sha,
+            )
+            adapters.append(adapter_info)
+
+    if model_name is not None:
+        adapters = [adapter for adapter in adapters if adapter.model_name == model_name]
+    return adapters
